//
//  CTAPIBaseManager.m
//  SocialSecurity
//
//  Created by 雷俊博 on 2019/7/15.
//  Copyright © 2019 KK. All rights reserved.
//

#import "CTAPIBaseManager.h"
#import "CTApiProxy.h"

@interface CTAPIBaseManager ()

@property (nonatomic, strong) AFHTTPRequestSerializer *httpRequestSerializer;

@property (nonatomic, strong) NSMutableArray *requestIdList;

@end

@implementation CTAPIBaseManager

#pragma mark - life cycle

- (instancetype)init
{
    self = [super init];
    if (self) {
        
    }
    return self;
}

- (void)dealloc
{
    [self cancelAllRequests];
    self.requestIdList = nil;
    NSLog(@"CTAPIBaseManager已销毁");
}
#pragma mark - NSCopying
- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

#pragma mark - public methods
- (void)cancelAllRequests
{
    [[CTApiProxy sharedInstance] cancelRequestWithRequestIDList:self.requestIdList];
    [self.requestIdList removeAllObjects];
}

- (void)cancelRequestWithRequestId:(NSInteger)requestID
{
    [self removeRequestIdWithRequestID:requestID];
    [[CTApiProxy sharedInstance] cancelRequestWithRequestID:@(requestID)];
}

- (void)loadDataWithUrl:url params:(NSDictionary *)params success:(void (^)(id responseObject))successCallback fail:(void (^)(NSError *))failCallback
{
    NSMutableURLRequest *request = [self.httpRequestSerializer requestWithMethod:@"GET" URLString:url parameters:params error:nil];
    
    NSNumber *requestId = [[CTApiProxy sharedInstance]callApiWithRequest:request success:^(id  _Nonnull response) {
        
        if (successCallback) {
            successCallback(response);
        }
    } fail:^(id  _Nonnull response) {
        if (failCallback) {
            failCallback(response);
        }
        
    }];
    [self.requestIdList addObject:requestId];
}

- (void)POSTWithUrl:(NSString *)url params:(NSDictionary *)params success:(void (^)(id responseObject))successCallback fail:(void (^)(NSError *))failCallback
{
    NSMutableURLRequest *request = [self.httpRequestSerializer requestWithMethod:@"POST" URLString:url parameters:params error:nil];
    
    NSNumber *requestId = [[CTApiProxy sharedInstance]callApiWithRequest:request success:^(id  _Nonnull response) {
        
        if (successCallback) {
            successCallback(response);
        }
    } fail:^(id  _Nonnull response) {
        if (failCallback) {
            failCallback(response);
        }
        
    }];
    [self.requestIdList addObject:requestId];
}



+(void)GET:(NSString *)url params:(NSDictionary *)params completionHandler:(void (^)(id responseObject, NSError *error))complete
{
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    
    [manager GET:url parameters:params progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        NSLog(@"JSON:%@",responseObject);
        complete(responseObject,nil);
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        NSLog(@"Error: %@",error);
        complete(nil,error);
    }];
    
}

+(void)GET2:(NSString *)url params:(NSDictionary *)params completionHandler:(void (^)(id responseObject, NSError *error))complete
{
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    
    manager.responseSerializer= [AFHTTPResponseSerializer serializer];
    manager.responseSerializer.acceptableContentTypes = [NSSet setWithObjects:@"application/json", @"text/json", @"text/javascript", @"text/html",@"image/jpeg",@"text/plain", nil];
    [manager GET:url parameters:params progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        NSLog(@"JSON:%@",responseObject);
        complete(responseObject,nil);
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        NSLog(@"Error: %@",error);
        complete(nil,error);
    }];
    
}

+(void)POST:(NSString *)url params:(NSDictionary *)params completionHandler:(void (^)(id responseObject, NSError *error))complete
{
   
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    
    manager.responseSerializer.acceptableContentTypes = [NSSet setWithObjects:@"application/json", @"text/json", @"text/javascript", @"text/html",@"image/jpeg",@"text/plain", nil];
   [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    
    
    [manager POST:url parameters:params progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        NSLog(@"JSON:%@",responseObject);
        complete(responseObject,nil);
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        NSLog(@"Error: %@",error);
        complete(nil,error);
    }];
}


+ (void)POST2:(NSString *)url params:(NSDictionary *)params completionHandler:(void (^)(id responseObject, NSError *error))complete
{
    AFHTTPSessionManager *manager = [AFHTTPSessionManager manager];
    [manager.requestSerializer setValue:@"application/x-www-form-urlencoded; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    manager.responseSerializer.acceptableContentTypes = [NSSet setWithObjects:@"text/xml",@"text/html", @"application/json", @"text/json", @"text/javascript", @"text/plain", nil];
    manager.requestSerializer = [AFHTTPRequestSerializer serializer];
    manager.responseSerializer = [AFHTTPResponseSerializer serializer];
    
    [manager POST:url parameters:params progress:nil success:^(NSURLSessionDataTask * _Nonnull task, id  _Nullable responseObject) {
        NSLog(@"JSON:%@",responseObject);
        complete(responseObject,nil);
    } failure:^(NSURLSessionDataTask * _Nullable task, NSError * _Nonnull error) {
        NSLog(@"Error: %@",error);
        complete(nil,error);
    }];
}


#pragma mark - private methods
- (void)removeRequestIdWithRequestID:(NSInteger)requestId
{
    NSNumber *requestIDToRemove = nil;
    for (NSNumber *storedRequestId in self.requestIdList) {
        if ([storedRequestId integerValue] == requestId) {
            requestIDToRemove = storedRequestId;
        }
    }
    if (requestIDToRemove) {
        [self.requestIdList removeObject:requestIDToRemove];
    }
}

#pragma mark - getters and setters
- (AFHTTPRequestSerializer *)httpRequestSerializer
{
    if (_httpRequestSerializer == nil) {
        _httpRequestSerializer = [AFHTTPRequestSerializer serializer];
        [_httpRequestSerializer setValue:@"application/json" forHTTPHeaderField:@"Content-Type"];
         [_httpRequestSerializer setValue:@"application/x-www-form-urlencoded; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    }
    return _httpRequestSerializer;
}
- (NSMutableArray *)requestIdList
{
    if (_requestIdList == nil) {
        _requestIdList = [[NSMutableArray alloc] init];
    }
    return _requestIdList;
}



@end
